require 'spec_helper'

describe 'puppet_enterprise::trapperkeeper::rbac' do
  before :all do
    @facter_facts = {
      'osfamily'          => 'Debian',
      'operatingsystem'   => 'Debian',
      'lsbmajdistrelease' => '6',
      'puppetversion'     => '3.6.2 (Puppet Enterprise 3.3.0)',
      'is_pe'             => 'true',
      'fqdn'              => 'rbac.rspec',
      'clientcert'        => 'awesomecert',
      'pe_concat_basedir'    => '/tmp/file',
    }

    @params = {
      'certname' => 'blah.foo.bar',
      'database_host' => 'db.rspec',
      'database_port' => 54321,
      'database_password' => 'secretly',
      'database_name' => 'pe-rbac',
    }

    @customization_params = {
      'password_reset_expiration' => 24,
      'session_timeout'           => 1440,
      'failed_attempts_lockout'   => 3,
      'ds_trust_chain'            => "/foo/bar",
    }
  end

  let(:facts) { @facter_facts }
  let(:params) { @params }
  let(:title) { 'rbac' }
  let(:config_file) {'/etc/puppetlabs/rbac/conf.d/rbac.conf'}
  let(:db_config_file) {'/etc/puppetlabs/rbac/conf.d/rbac-database.conf'}

  context "when the parameters are valid" do
    it { should contain_file(config_file).with_content(%r[certificate-whitelist:\s*"/etc/puppetlabs/rbac/rbac-certificate-whitelist]) }
    it { should contain_file(config_file).with_content(%r[token-private-key:\s*"/opt/puppet/share/rbac/certs/blah.foo.bar.private_key.pem]) }
    it { should contain_file(config_file).with_content(%r[token-public-key:\s*"/opt/puppet/share/rbac/certs/blah.foo.bar.cert.pem]) }
    it { should contain_file(config_file).with(:owner => "pe-rbac",
                                                                           :group => "pe-rbac",
                                                                           :mode => "0640") }

    it { should contain_file(config_file).without_content(%r[password-reset-expiration:]) }
    it { should contain_file(config_file).without_content(%r[session-timeout:]) }
    it { should contain_file(config_file).without_content(%r[failed-attempts-lockout:]) }
    it { should contain_file(config_file).without_content(%r[ds-trust-chain:]) }

    it { should contain_file(db_config_file).with_content(%r[subname:.*//db\.rspec:54321/pe-rbac]) }
    it { should contain_pe_concat__fragment('rbac rbac-service') }
    it { should contain_pe_concat__fragment('rbac rbac-storage-service') }
    it { should contain_pe_concat__fragment('rbac rbac-http-api-service') }
    it { should contain_pe_concat__fragment('rbac activity-reporting-service') }
    it { should contain_pe_concat__fragment('rbac jetty9-service') }
  end

  context "using customization parameters" do
    let(:params) { @params.merge(@customization_params) }

    it { should contain_file(config_file).with_content(%r[password-reset-expiration:\s*24]) }
    it { should contain_file(config_file).with_content(%r[session-timeout:\s*1440]) }
    it { should contain_file(config_file).with_content(%r[failed-attempts-lockout:\s*3]) }
    it { should contain_file(config_file).with_content(%r[ds-trust-chain:\s*/foo/bar]) }
  end
end
